#ifndef ACCEL_H
#define ACCEL_H

#include <QObject>
#include <QTimer>

#include <math.h>
#include <stdio.h>

#include <iostream>

#define TILT 600
#define ACCEL_FILENAME "/sys/class/i2c-adapter/i2c-3/3-001d/coord"

class Accelerometer : public QObject {
    Q_OBJECT
private:
    int x;
    int y;
    int z;
    bool initial;
    QTimer* timer;

signals:
    void accelHit();

public:
    Accelerometer(QObject* parent) : QObject(parent), x(0), y(0), z(0), initial(true) {
        timer = new QTimer();
        QObject::connect(timer, SIGNAL(timeout()),
                         this, SLOT(update()));
        timer->start(500);
    }

public slots:
    void update() {
        std::cout << "Trying to read accel." << std::endl;
        int a, b, c;
        FILE* fp = fopen(ACCEL_FILENAME, "r");
        if (fp != NULL) {
            if (fscanf(fp, "%i %i %i", &a, &b, &c) == 3) {
                if (!initial && (fabs(a-x) > TILT ||
                                 fabs(b-y) > TILT ||
                                 fabs(c-z) > TILT)) {
                    std::cout << "HIT detected" << std::endl;
                    std::cout << "A <-> X: " << a << " " << x << std::endl;
                    std::cout << "B <-> Y: " << b << " " << y << std::endl;
                    std::cout << "C <-> Z: " << c << " " << z << std::endl;
                    accelHit();
                } else {
                    std::cout << "no hit. readings: " << a << ", " << b << ", " << c << std::endl;
                }
                x = a;
                y = b;
                z = c;
                initial = false;
            } else {
                std::cout << "Error reading accel data." << std::endl;
            }
            fclose(fp);
        } else {
            std::cout << "Error opening accel: " << ACCEL_FILENAME << std::endl;
        }
    }
};

#endif // ACCEL_H
