#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>

#include <QGraphicsScene>
#include <QGraphicsLineItem>
#include <QGraphicsEllipseItem>
#include <QList>
#include <QKeyEvent>
#include <QPointF>

#include <iostream>

#include "wheel.h"
#include "accel.h"

enum SketchMode {
    SKETCH_MODE_HORIZONTAL = 0,
    SKETCH_MODE_VERTICAL,
    SKETCH_MODE_INVALID,
};

namespace Ui
{
    class MainWindow;
}

#define STEP 10
#define DOTT 3

#define WHEEL 160
#define BORDER 10

#define WIDTH 700
#define HEIGHT 350

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void keyPressEvent(QKeyEvent* event) {
        QMainWindow::keyPressEvent(event);
        switch (event->key()) {
            case Qt::Key_A:
                go(SKETCH_MODE_HORIZONTAL, -STEP);
                break;
            case Qt::Key_D:
                go(SKETCH_MODE_HORIZONTAL, STEP);
                break;
            case Qt::Key_W:
                go(SKETCH_MODE_VERTICAL, -STEP);
                break;
            case Qt::Key_S:
                go(SKETCH_MODE_VERTICAL, STEP);
                break;
            case Qt::Key_Q:
                shiver();
                break;
        }
    }

private:
    void shiver() {
        QGraphicsLineItem* item;
        foreach (item, lines) {
            QPen pen = item->pen();
            switch (pen.style()) {
                case Qt::SolidLine:
                    pen.setStyle(Qt::DashLine);
                    break;
                case Qt::DashLine:
                    pen.setStyle(Qt::DashDotLine);
                    break;
                case Qt::DashDotLine:
                    pen.setStyle(Qt::DashDotDotLine);
                    break;
                case Qt::DashDotDotLine:
                    pen.setStyle(Qt::DotLine);
                    break;
                default:
                    lines.removeOne(item);
                    scene->removeItem(item);
                    break;
            }
            item->setPen(pen);
        }
        std::cout << "Items left after shivering: " << lines.count() << std::endl;
        sketchMode = SKETCH_MODE_INVALID;
    }

    void go(SketchMode newMode, qreal direction) {
        qreal x2, y2;

        if ((sketchMode != newMode) ||
            (lastDirection > 0 && direction < 0) ||
            (lastDirection < 0 && direction > 0)) {
            currentLine = new QGraphicsLineItem(currentPosition.x(), currentPosition.y(),
                                                currentPosition.x(), currentPosition.y());
            lines.append(currentLine);
            scene->addItem(currentLine);
            std::cout << "Adding line" << std::endl;
            sketchMode = newMode;
            lastDirection = direction;
        }

        x2 = currentLine->line().x2();
        y2 = currentLine->line().y2();

        switch (sketchMode) {
            case SKETCH_MODE_HORIZONTAL:
                x2 += direction;
                break;
            case SKETCH_MODE_VERTICAL:
                y2 += direction;
                break;
            default:
                return;
        }

        currentLine->setLine(currentLine->line().x1(),
                             currentLine->line().y1(),
                             x2,
                             y2);

        currentPosition.setX(x2);
        currentPosition.setY(y2);
        inputDot->setRect(x2-DOTT, y2-DOTT, DOTT*2, DOTT*2);
    }

    Ui::MainWindow *ui;

    SketchMode sketchMode;
    QGraphicsScene* scene;
    QGraphicsLineItem* currentLine;
    QList<QGraphicsLineItem*> lines;
    QPointF currentPosition;
    QGraphicsEllipseItem* inputDot;
    Wheel* horizontalWheel;
    Wheel* verticalWheel;
    int lastDirection;
    Accelerometer* accel;

public slots:
    void rotateHorizontal(qreal angle) {
        go(SKETCH_MODE_HORIZONTAL, angle);
    }

    void rotateVertical(qreal angle) {
        go(SKETCH_MODE_VERTICAL, angle);
    }

    void shiverScreen() {
        shiver();
    }
};

#endif // MAINWINDOW_H
