#ifndef WHEEL_H
#define WHEEL_H

#include <QGraphicsEllipseItem>
#include <QGraphicsSceneMouseEvent>
#include <QPointF>

#include <iostream>

#include <math.h>

class Wheel : public QObject, public QGraphicsEllipseItem
{
    Q_OBJECT
private:
    bool turning;
    QPointF lastPos;
    qreal lastAngle;
    qreal angle;

public:
    Wheel(qreal x, qreal y, qreal w, qreal h);

    void mousePressEvent(QGraphicsSceneMouseEvent* event) {
        turning = true;
        lastPos = event->pos();
        angle = atan2(lastPos.y(), lastPos.x()) + M_PI;
        std::cout << "ANGLE: " << angle*180/M_PI << std::endl;
        std::cout << "Mouse press event (" << lastPos.x() << ", " << lastPos.y() << ")" << std::endl;
        lastAngle = angle;
    }

    void mouseReleaseEvent(QGraphicsSceneMouseEvent* event) {
        std::cout << "Mouse release event" << std::endl;
        (void)event;
        turning = false;
    }

    void mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
        qreal diff;
        std::cout << "MOVE(" << (lastPos.x()-event->pos().x()) << ")" << std::endl;

        lastPos = event->pos();
        angle = atan2(lastPos.y(), lastPos.x()) + M_PI;
        diff = angle - lastAngle;

        if (diff > M_PI) {
            diff -= 2*M_PI;
        } else if (diff < -M_PI) {
            diff += 2*M_PI;
        }

        lastAngle = angle;

        std::cout << "ANGLE: " << diff*180/M_PI << std::endl;
        rotated(diff*180/M_PI);
    }

signals:
    void rotated(qreal angle);
};

#endif // WHEEL_H
